/**
 * Created by CCristi <ccovali@mitocgroup.com> on 4/19/16.
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FrontendEvent = undefined;

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

var _AbstractEvent = require('./AbstractEvent');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Frontend Event
 */
class FrontendEvent extends _AbstractEvent.AbstractEvent {
  /**
   * @param {String} args
   */
  constructor(...args) {
    super(...args);
  }

  /**
   * @returns {String}
   */
  getEventLevel() {
    return _AbstractEvent.AbstractEvent.FRONTEND_EVENT_LEVEL;
  }

  /**
   * @returns {Object}
   */
  toJSON() {
    return this._data;
  }

  /**
   * @returns {Object}
   */
  getValidationSchema() {
    return _joi2.default.object().keys({
      eventLevel: _joi2.default.string().required().allow([_AbstractEvent.AbstractEvent.FRONTEND_EVENT_LEVEL]),
      engine: _joi2.default.string().required().allow(FrontendEvent.ENGINES),
      time: _joi2.default.number().integer().required(),
      context: _joi2.default.string().required().allow([_AbstractEvent.AbstractEvent.FRONTEND_CONTEXT]),
      environment: _joi2.default.object().unknown().optional().default({}),
      identityId: _joi2.default.string().required(),
      memoryUsage: _joi2.default.object().unknown().optional().default({}),
      metadata: _joi2.default.object().unknown().optional().default({}),
      location: _joi2.default.string().optional()
    });
  }

  /**
   * @param {Object} event
   * @returns {Object}
   * @private
   */
  _enrichWithContextData(event) {
    event = super._enrichWithContextData(event);
    event.location = typeof window !== 'undefined' ? `${window.location.pathname}${window.location.hash}` : null;

    return event;
  }

  /**
   * @returns {String[]}
   */
  static get ENGINES() {
    return ['angular', 'vanilla'];
  }
}
exports.FrontendEvent = FrontendEvent;