/**
 * Created by AlexanderC on 6/15/15.
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RavenBrowserDriver = undefined;

var _AbstractDriver = require('./AbstractDriver');

var _Log = require('../Log');

var _raven = require('raven');

var _raven2 = _interopRequireDefault(_raven);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Raven/Sentry logging for browser
 *
 * @todo replace it on compile time rather than runtime!
 */
class RavenBrowserDriver extends _AbstractDriver.AbstractDriver {
  /**
   * @param {String} dsn
   * @param {Object} options
   */
  constructor(dsn, options = {}) {
    super();

    options = _util2.default._extend(options, {
      maxMessageLength: 256
    });

    _raven2.default.config(RavenBrowserDriver._prepareDsn(dsn), options).install();
  }

  /**
   * @todo: tmp hook, remove it before persisting frontend config
   *
   * Remove password from url to not expose it into browser
   *
   * @param {String} dsn
   * @returns {String}
   */
  static _prepareDsn(dsn) {
    let parsedDsn = _url2.default.parse(dsn);
    parsedDsn.auth = parsedDsn.auth.split(':')[0];

    return _url2.default.format(parsedDsn);
  }

  /**
   * @param {String} msg
   * @param {String} level
   * @param {*} context
   */
  log(msg, level, context) {
    let nativeLevel = RavenBrowserDriver._mapLevel(level);

    _raven2.default.captureMessage(msg, {
      level: nativeLevel,
      extra: context,
      tags: {
        originalLevel: level
      }
    });
  }

  /**
   * @param {String} level
   * @returns {string}
   * @private
   */
  static _mapLevel(level) {
    let nativeLevel = 'info';

    switch (level) {
      case _Log.Log.EMERGENCY:
      case _Log.Log.CRITICAL:
        nativeLevel = 'fatal';
        break;
      case _Log.Log.ALERT:
      case _Log.Log.WARNING:
      case _Log.Log.NOTICE:
        nativeLevel = 'warning';
        break;
      case _Log.Log.ERROR:
        nativeLevel = 'error';
        break;
      case _Log.Log.INFO:
        nativeLevel = 'info';
        break;
      case _Log.Log.DEBUG:
        nativeLevel = 'debug';
        break;
    }

    return nativeLevel;
  }
}
exports.RavenBrowserDriver = RavenBrowserDriver;