/**
 * Created by AlexanderC on 6/15/15.
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RavenDriver = undefined;

var _AbstractDriver = require('./AbstractDriver');

var _Log = require('../Log');

var _raven = require('raven');

var _raven2 = _interopRequireDefault(_raven);

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Raven/Sentry logging
 */
class RavenDriver extends _AbstractDriver.AbstractDriver {
  /**
   * @param {String} dsn
   * @param {Object} options
   */
  constructor(dsn, options = {}) {
    super();

    options = _util2.default._extend(options, {
      maxMessageLength: 256
    });

    this._clients = {};

    for (let levelKey in _Log.Log.LEVELS) {
      if (!_Log.Log.LEVELS.hasOwnProperty(levelKey)) {
        continue;
      }

      let nativeLevel = RavenDriver._mapLevel(_Log.Log.LEVELS[levelKey]);

      options.level = nativeLevel;

      this._clients[nativeLevel] = new _raven2.default.Client(dsn, options);
    }
  }

  /**
   * @returns {Raven.Client[]}
   */
  get clients() {
    return this._clients;
  }

  /**
   * @param {String} msg
   * @param {String} level
   * @param {*} context
   */
  log(msg, level, context) {
    let nativeLevel = RavenDriver._mapLevel(level);

    this._clients[nativeLevel].captureMessage(msg, {
      extra: context,
      tags: {
        originalLevel: level
      }
    });
  }

  /**
   * @param {String} level
   * @returns {string}
   * @private
   */
  static _mapLevel(level) {
    let nativeLevel = 'info';

    switch (level) {
      case _Log.Log.EMERGENCY:
      case _Log.Log.CRITICAL:
        nativeLevel = 'fatal';
        break;
      case _Log.Log.ALERT:
      case _Log.Log.WARNING:
      case _Log.Log.NOTICE:
        nativeLevel = 'warning';
        break;
      case _Log.Log.ERROR:
        nativeLevel = 'error';
        break;
      case _Log.Log.INFO:
        nativeLevel = 'info';
        break;
      case _Log.Log.DEBUG:
        nativeLevel = 'debug';
        break;
    }

    return nativeLevel;
  }
}
exports.RavenDriver = RavenDriver;